<?php declare(strict_types=1);

namespace Micropayment\Helper;

use Exception;
use Shopware\Core\Checkout\Customer\CustomerDefinition;
use Shopware\Core\Checkout\Order\OrderDefinition;
use Shopware\Core\Framework\Context;
use Shopware\Core\Framework\DataAbstractionLayer\EntityRepository;
use Shopware\Core\Framework\Uuid\Uuid;
use Shopware\Core\System\CustomField\CustomFieldTypes;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Shopware\Core\Framework\Plugin\Context\UninstallContext;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Criteria;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Filter\EqualsAnyFilter;
use Shopware\Core\Framework\DataAbstractionLayer\Search\IdSearchResult;

class CustomFieldHelper
{
    /** @var ContainerInterface */
    private $container;

    /** @var EntityRepository */
    private $customFieldSetRepository;

    /**
     * CustomFieldService constructor.
     *
     * @param ContainerInterface $container
     * @param EntityRepository $customFieldSetRepository
     */
    public function __construct(
        $container,
        EntityRepository $customFieldSetRepository
    )
    {
        $this->container = $container;
        $this->customFieldSetRepository = $customFieldSetRepository;
    }

    public function addCustomFields(Context $context)
    {
        try {
//            $customFieldSetId = '9B9ABE2ED6ED47D998DF4CADB60BEA93'; //Uuid::randomHex();
//            $authIdFieldId = 'AF2BA2970A574ED8806D48C2D1AEB33A'; //Uuid::randomHex();
//            $trxIdFieldId = '8FDAB3ED6CD947B180574FA5377C4850'; //Uuid::randomHex();
//            $commentFieldId = 'af6bec46ea914eb1bec06fa563b7a6c4'; //Uuid::randomHex();

            $customFieldSetId = Uuid::randomHex();
            $authIdFieldId = Uuid::randomHex();
            $trxIdFieldId = Uuid::randomHex();
            $commentFieldId = Uuid::randomHex();

            $data = [[
                'Id' => $customFieldSetId,
                'name' => 'micropayment',
                'config' => [
                    'label' => [
                        'de-DE' => 'Micropayment',
                        'en-GB' => 'Micropayment'
                    ]
                ],
                'customFields' => [
                    [
                        'Id' => $authIdFieldId,
                        'name' => 'mcpay_authid',
                        'type' => CustomFieldTypes::TEXT,
                        'config' => [
                            'componentName' => 'sw-field',
                            'customFieldType' => CustomFieldTypes::TEXT,
                            'customFieldPosition' => 10,
                            'label' => [
                                'en-GB' => 'Auth ID',
                                'de-DE' => 'Auth ID'
                            ]
                        ]
                    ],
                    [
                        'Id' => $trxIdFieldId,
                        'name' => 'mcpay_trxid',
                        'type' => CustomFieldTypes::TEXT,
                        'config' => [
                            'componentName' => 'sw-field',
                            'customFieldType' => CustomFieldTypes::TEXT,
                            'customFieldPosition' => 20,
                            'label' => [
                                'en-GB' => 'Transaction ID',
                                'de-DE' => 'Transaktions ID'
                            ]
                        ]
                    ],
                    [
                        'Id' => $commentFieldId,
                        'name' => 'mcpay_comments',
                        'type' => CustomFieldTypes::HTML,
                        'config' => [
                          'componentName' => 'sw-text-editor',
                            'customFieldType' => "textEditor",
                            'customFieldPosition' => 30,
                            'label' => [
                                'en-GB' => 'Comments',
                                'de-DE' => 'Kommentare'
                            ]
                        ]
                    ]
                ],
                'relations' => [
                    [
                        'Id' => $authIdFieldId,
                        'entityName' => OrderDefinition::ENTITY_NAME
                    ],
                    [
                        'Id' => $trxIdFieldId,
                        'entityName' => CustomerDefinition::ENTITY_NAME
//                    ],
//                    [
//                        'Id' => $commentFieldId,
//                        'entityName' => OrderDefinition::ENTITY_NAME
                    ]
                ]
            ]];
            //echo '<pre>'.print_r($data,true).'</pre>';
            $this->customFieldSetRepository->upsert($data, $context);
        } catch (Exception $e) {
            echo 'ERROR:'.$e->getMessage();
        }
    }

  /**
   * removeCustomField
   *
   * @param \Shopware\Core\Framework\Plugin\Context\UninstallContext $uninstallContext
   */
  public function removeCustomField(UninstallContext $uninstallContext)
  {
    $fieldIds = $this->customFieldsExist($uninstallContext->getContext());

    if ($fieldIds) {
      $this->customFieldSetRepository->delete(array_values($fieldIds->getData()), $uninstallContext->getContext());
    }
  }

  /**
   * customFieldsExist
   *
   * @param \Shopware\Core\Framework\Context $context
   *
   * @return Shopware\Core\Framework\DataAbstractionLayer\Search\IdSearchResult|null
   */
  private function customFieldsExist(Context $context): ?IdSearchResult
  {
    $criteria = new Criteria();
    //$criteria->addFilter(new EqualsAnyFilter('name', ['micropayment','mcpay_authid','mcpay_trxid','mcpay_comments']));
    $criteria->addFilter(new EqualsAnyFilter('name', ['micropayment']));

    $ids = $this->customFieldSetRepository->searchIds($criteria, $context);

    return $ids->getTotal() > 0 ? $ids : null;
  }
}
