<?php

namespace Micropayment\Helper;

use Shopware\Core\Checkout\Cart\Exception\OrderNotFoundException;
use Shopware\Core\Checkout\Order\Aggregate\OrderTransaction\OrderTransactionEntity;
use Shopware\Core\Checkout\Order\Aggregate\OrderTransaction\OrderTransactionStateHandler;
use Shopware\Core\Checkout\Order\OrderEntity;
use Shopware\Core\Checkout\Payment\Exception\InvalidOrderException;
use Shopware\Core\Framework\Context;
use Shopware\Core\Framework\DataAbstractionLayer\EntityRepository;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Criteria;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Filter\EqualsFilter;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Sorting\FieldSorting;
use Shopware\Core\System\StateMachine\StateMachineRegistry;
use Shopware\Core\Framework\Plugin\PluginEntity;

class PaymentStatusHelper
{
  /** @var OrderTransactionStateHandler */
  protected $orderTransactionStateHandler;

  /** @var StateMachineRegistry */
  protected $stateMachineRegistry;

  /** @var \Shopware\Core\Framework\DataAbstractionLayer\EntityRepository */
  protected $orderRepository;

  protected $pluginRepository;

  protected $shopVersion;

  /**
   * PaymentStatusHelper constructor.
   *
   * @param \Shopware\Core\Checkout\Order\Aggregate\OrderTransaction\OrderTransactionStateHandler $orderTransactionStateHandler
   * @param \Shopware\Core\System\StateMachine\StateMachineRegistry                               $stateMachineRegistry
   * @param \Shopware\Core\Framework\DataAbstractionLayer\EntityRepository               $orderRepository
   * @param \Shopware\Core\Framework\DataAbstractionLayer\EntityRepository               $pluginRepository
   * @param \Shopware\Core\System\SystemConfig\SystemConfigService                                $sysConfig
   */
  public function __construct(
    OrderTransactionStateHandler $orderTransactionStateHandler,
    StateMachineRegistry         $stateMachineRegistry,
    EntityRepository    $orderRepository,
    EntityRepository    $pluginRepository,
    string                       $shopVersion
  )
  {
    $this->orderTransactionStateHandler = $orderTransactionStateHandler;
    $this->stateMachineRegistry         = $stateMachineRegistry;
    $this->orderRepository              = $orderRepository;
    $this->pluginRepository             = $pluginRepository;
    $this->shopVersion                  = $shopVersion;
  }

  /**
   * setState
   *
   * @param $trxid
   * @param $state
   * @param $context
   *
   * @return mixed
   */
  public function setState($trxid, $state, $context)
  {
    return $this->orderTransactionStateHandler->$state($trxid, $context);
  }

  /**
   * getOrderTransactionByOrderNumber
   *
   * @param string                           $orderNumber
   * @param \Shopware\Core\Framework\Context $context
   * @param string                           $trxId
   *
   * @return \Shopware\Core\Checkout\Order\Aggregate\OrderTransaction\OrderTransactionEntity
   */
  public function getOrderTransactionByOrderNumber(string $orderNumber, Context $context, string $trxId = ''): OrderTransactionEntity
  {
    $criteria = new Criteria();
    $criteria->addAssociation('transactions');
    $criteria->getAssociation('transactions')->addSorting(new FieldSorting('createdAt'));
    $criteria->addFilter(new EqualsFilter('orderNumber', $orderNumber));

    /** @var OrderEntity|null $order */
    $order = $this->orderRepository->search($criteria, $context)->first();
    if ($order === NULL) {
      throw new OrderNotFoundException($orderNumber);
    }

    $transactionCollection = $order->getTransactions();
    if ($transactionCollection === NULL) {
      throw new InvalidOrderException($orderNumber);
    }

    if (!empty($trxId)) {
      $transaction = $transactionCollection->get($trxId);
    } else {
      $transaction = $transactionCollection->last();
    }
    if ($transaction === NULL) {
      throw new InvalidOrderException($orderNumber);
    }

    return $transaction;
  }

  /**
   * getOrderTransaction
   *
   * @param string                           $orderId
   * @param \Shopware\Core\Framework\Context $context
   *
   * @return \Shopware\Core\Checkout\Order\Aggregate\OrderTransaction\OrderTransactionEntity
   */
  public function getOrderTransaction(string $orderId, Context $context): OrderTransactionEntity
  {
    $criteria = new Criteria([$orderId]);
    $criteria->addAssociation('transactions');
    $criteria->getAssociation('transactions')->addSorting(new FieldSorting('createdAt'));
    /** @var OrderEntity|null $order */
    $order = $this->orderRepository->search($criteria, $context)->first();

    if ($order === NULL) {
      throw new OrderNotFoundException($orderId);
    }

    $transactionCollection = $order->getTransactions();

    if ($transactionCollection === NULL) {
      throw new InvalidOrderException($orderId);
    }

    $transaction = $transactionCollection->last();

    if ($transaction === NULL) {
      throw new InvalidOrderException($orderId);
    }

    return $transaction;
  }

//    private function createOrderTransaction(
//      string $orderId,
//      string $stateId,
//      string $paymentMethod,
//      float $amount,
//      Context $context
//    ): void {
//      $orderTransactionId = Uuid::randomHex();
//      $transaction = [
//        'id' => $orderTransactionId,
//        'orderId' => $orderId,
//        'stateId' => $stateId,
//        'paymentMethodId' => $paymentMethod,
//        'amount' => new CalculatedPrice(
//          $amount,
//          $amount,
//          new CalculatedTaxCollection(),
//          new TaxRuleCollection(),
//          1
//        )
//      ];
//      if (!$this->orderTransactionRepository->create([$transaction], $context)) {
//        return;
//      }
//    }

  /**
   * getPluginDetails
   *
   * @param string                           $name
   * @param \Shopware\Core\Framework\Context $context
   *
   * @return \Shopware\Core\Framework\Plugin\PluginEntity
   */
  public function getPluginDetails(string $name, Context $context): PluginEntity
  {
    $criteria = new Criteria([]);
    $criteria->addFilter(new EqualsFilter('name', $name));
    /** @var OrderEntity|null $order */
    $data = $this->pluginRepository->search($criteria, $context)->first();
    return $data;
  }

  /**
   * getShopVersion
   *
   * @return string
   */
  public function getShopVersion()
  {
    return $this->shopVersion;
  }

  /**
   * setCustomFields
   *
   * @param $orderId
   * @param $fields
   * @param $context
   */
  public function setCustomFields($orderId, $fields, $context)
  {
    $criteria = new Criteria();
    $criteria->addFilter(new EqualsFilter('orderNumber', $orderId));
    /** @var OrderEntity|null $order */
    $order = $this->orderRepository->search($criteria, $context)->first();
    //echo $order->getId().' found';
    if ($order === NULL) {
      throw new OrderNotFoundException($orderId);
    }
    //$order->setCustomFields($fields);
    $old = $order->getCustomFields();
    //echo '<pre>'.print_r($old,true).'</pre>'; exit();
    if (!empty($old['mcpay_comments'])) {
      $fields['mcpay_comments'] = $fields['mcpay_comments']."<br>\n".$old['mcpay_comments'];
    } else {
      $fields['mcpay_comments'] = $fields['mcpay_comments'];
    }

    $data = [[
               'id'           => $order->getId(),
               'customFields' => $fields,
             ]];
    $this->orderRepository->update($data, $context);
  }

}
