<?php

namespace Micropayment\Service\Method;

use Micropayment\Helper\McPayment;
use Micropayment\Service\Config;
use Micropayment\Service\ApiPay;
use Shopware\Core\Checkout\Order\Aggregate\OrderTransaction\OrderTransactionStateHandler;
use Shopware\Core\Checkout\Payment\Cart\AsyncPaymentTransactionStruct;
use Shopware\Core\Checkout\Payment\Exception\AsyncPaymentProcessException;
use Shopware\Core\Framework\DataAbstractionLayer\EntityRepository;
use Shopware\Core\Framework\Validation\DataBag\RequestDataBag;
use Shopware\Core\System\SalesChannel\SalesChannelContext;
use Symfony\Component\HttpFoundation\RedirectResponse;

class CreditcardApi extends ApiPay
{
  public const PAYMENT_METHOD_NAME = 'CCARD_API';

  /**
   * CreditcardApi constructor.
   *
   * @param \Shopware\Core\Checkout\Order\Aggregate\OrderTransaction\OrderTransactionStateHandler $transactionStateHandler
   * @param \Micropayment\Service\Config                                                          $config
   * @param \Micropayment\Helper\McPayment                                                        $mcp
   * @param \Shopware\Core\Framework\DataAbstractionLayer\EntityRepository               $localeRepo
   */
  public function __construct(
    OrderTransactionStateHandler $transactionStateHandler,
    Config                       $config,
    McPayment                    $mcp,
    EntityRepository    $localeRepo
  )
  {
    $this->paymethod = self::PAYMENT_METHOD_NAME;
    parent::__construct($transactionStateHandler, $config, $mcp, $localeRepo);
  }

  /**
   * pay
   *
   * @param \Shopware\Core\Checkout\Payment\Cart\AsyncPaymentTransactionStruct $transaction
   * @param \Shopware\Core\Framework\Validation\DataBag\RequestDataBag         $dataBag
   * @param \Shopware\Core\System\SalesChannel\SalesChannelContext             $salesChannelContext
   *
   * @return \Symfony\Component\HttpFoundation\RedirectResponse
   */
  public function pay(
    AsyncPaymentTransactionStruct $transaction,
    RequestDataBag                $dataBag,
    SalesChannelContext           $salesChannelContext
  ): RedirectResponse
  {
    try {
      //dd($_REQUEST);
      if (empty($_REQUEST['McPay_ccToken'])){
        throw new AsyncPaymentProcessException(
          $transaction->getOrderTransaction()->getId(),
          'Token cant be empty!'.PHP_EOL
        );
      }

      $settings = $this->config->getSettings($salesChannelContext->getSalesChannel()->getId());
      $config = $this->mcp->init($settings);

      $order    = $transaction->getOrder();
      $userData = $this->mcp->getUserData($order, $settings, $salesChannelContext);

      $langISO             = $this->getLocaleCodeFromContext($order->getLanguageId(), $salesChannelContext);
      $userData['lang']    = $langISO;
      $orderID             = $order->getId();
      $trxID               = $transaction->getOrderTransaction()->getId();
      $userData['trxid']   = $trxID;
      $userData['oid']     = $order->getOrderNumber(); // $order->getId();
      $userData['respurl'] = urlencode($transaction->getReturnUrl()); // final thank you url

      $notifyURL = $this->getURL($salesChannelContext, $config['testMode'], 'notify');
      $errorURL = $this->getURL($salesChannelContext, FALSE, 'error').$orderID.'?error-code=CHECKOUT__ASYNC_PAYMENT_PROCESS_INTERRUPTED';

      $query = '&orderid='.$userData['oid'].'&trxid='.$trxID
        .'&returnURL='.urlencode($userData['respurl'])
        .'&errorURL='.urlencode(urlencode($errorURL));
      $config['returnURL'] = $notifyURL.'?function=handle3D'.$query;
      $config['returnURLNO3D'] = $notifyURL.'?function=handleNO3D'.$query;
      $config['autogenSession'] = TRUE;

      $payData = array(
        'token'        => $_REQUEST['McPay_ccToken'],
        'holder'       => $_REQUEST['McPay_ccHOLDER'],
        'pan'          => $_REQUEST['McPay_ccPAN'],
        'month'        => $_REQUEST['McPay_ccMonth'],
        'year'         => $_REQUEST['McPay_ccYear'],
        'remember'     => $_REQUEST['McPay_ccREMEMBER'],
        'force3d'      => '1',
        'forceNewCard' => '0',  // if new card chosen, we really need to use the new card. without this param he always uses the old card i.e. without 3d Secure!!!
      );
      if (!empty($_REQUEST['McPay_ccReuse']) && $_REQUEST['McPay_ccReuse'] == 'new'){
        $payData['forceNewCard'] = 1;
      }
      if (!empty($userData['guest'])){
        $payData['forceNewCard'] = 1; // guest uses always new cards
      }

      $res   = $this->mcp->bookCCard($userData, $config, $payData);
      if ($res->trxInfo->secureStatus != 'START3D') {
        // call notify internal
        $post = array(
          'function' => 'billing',
          'orderid'  => $userData['order_id'],
          'auth'     => $res->trxInfo->transactionId,
          'amount'   => $userData['amount'],
          'currency' => $userData['currency'],
          'testmode' => $config['testMode'],
          'intresp'  => 1,
          'trxid'    => $trxID,
        );

        $res  = $this->mcp->doCall($notifyURL, $post); // call NotifyController
        //dd($res);
        if ($this->mcp->debug && !empty($res)) $this->mcp->log->log(__CLASS__.'->'.__FUNCTION__, print_r($res, true)); // log error

        // jump to thank you page
        $redirectUrl = urldecode($userData['respurl']);
      } else if (!empty($res->forward)) {
        $redirectUrl = $res->forwardURL;
      }

    } catch (\Exception $e) {
      if ($this->mcp->debug) $this->mcp->log->log(__CLASS__.'->'.__FUNCTION__, $e->getMessage().' - '.$e->getCode()); // log error
      if ($this->mcp->debug && !empty($res)) $this->mcp->log->log(__CLASS__.'->'.__FUNCTION__, print_r($res, true)); // log error

      throw new AsyncPaymentProcessException(
        $transaction->getOrderTransaction()->getId(),
        'An error occurred during the communication with external payment gateway'.PHP_EOL.$e->getMessage()
      );
    }

    // Redirect to external gateway
    return new RedirectResponse($redirectUrl);
  }
}
