<?php

namespace Micropayment\Service\Method;

use Micropayment\Helper\McPayment;
use Micropayment\Service\Config;
use Micropayment\Service\ApiPay;
use Micropayment\Helper\PaymentStatusHelper;
use Shopware\Core\Checkout\Order\Aggregate\OrderTransaction\OrderTransactionStateHandler;
use Shopware\Core\Checkout\Payment\Cart\AsyncPaymentTransactionStruct;
use Shopware\Core\Checkout\Payment\Exception\AsyncPaymentProcessException;
use Shopware\Core\Framework\DataAbstractionLayer\EntityRepository;
use Shopware\Core\Framework\Validation\DataBag\RequestDataBag;
use Shopware\Core\System\SalesChannel\SalesChannelContext;
use Symfony\Component\HttpFoundation\RedirectResponse;

class PrepayApi extends ApiPay
{
  public const PAYMENT_METHOD_NAME = 'PREPAY_API';

  /** @var \Micropayment\Helper\PaymentStatusHelper */
  protected $psh;

  /**
   * @param \Shopware\Core\Checkout\Order\Aggregate\OrderTransaction\OrderTransactionStateHandler $transactionStateHandler
   * @param \Micropayment\Service\Config                                                          $config
   * @param \Micropayment\Helper\McPayment                                                        $mcp
   * @param \Shopware\Core\Framework\DataAbstractionLayer\EntityRepository               $localeRepo
   * @param \Micropayment\Helper\PaymentStatusHelper                                              $psh
   */
  public function __construct(
    OrderTransactionStateHandler $transactionStateHandler,
    Config                       $config,
    McPayment                    $mcp,
    EntityRepository    $localeRepo,
    PaymentStatusHelper          $psh
  )
  {
    $this->paymethod = self::PAYMENT_METHOD_NAME;
    $this->psh       = $psh;
    parent::__construct($transactionStateHandler, $config, $mcp, $localeRepo);
  }

  /**
   * pay
   *
   * @param \Shopware\Core\Checkout\Payment\Cart\AsyncPaymentTransactionStruct $transaction
   * @param \Shopware\Core\Framework\Validation\DataBag\RequestDataBag         $dataBag
   * @param \Shopware\Core\System\SalesChannel\SalesChannelContext             $salesChannelContext
   *
   * @return \Symfony\Component\HttpFoundation\RedirectResponse
   */
  public function pay(
    AsyncPaymentTransactionStruct $transaction,
    RequestDataBag                $dataBag,
    SalesChannelContext           $salesChannelContext
  ): RedirectResponse
  {
    try {
      //dd($_REQUEST);

      $settings              = $this->config->getSettings($salesChannelContext->getSalesChannel()->getId());
      $settings['paymethod'] = 'prepay';
      $config                = $this->mcp->init($settings);
      if ($this->mcp->debug) $this->mcp->log->log(__CLASS__.'->'.__FUNCTION__, print_r($_REQUEST, TRUE)); // log error

      $order    = $transaction->getOrder();
      $userData = $this->mcp->getUserData($order, $settings, $salesChannelContext);

      $langISO             = $this->getLocaleCodeFromContext($order->getLanguageId(), $salesChannelContext);
      $userData['lang']    = $langISO;
      $orderID             = $order->getId();
      $trxID               = $transaction->getOrderTransaction()->getId();
      $userData['trxid']   = $trxID;
      $userData['oid']     = $order->getOrderNumber(); // $order->getId();
      $userData['respurl'] = urlencode($transaction->getReturnUrl()); // final thank you url

      $config['autogenSession'] = TRUE;
      $errorURL = $this->getURL($salesChannelContext, FALSE, 'error').$orderID.'?error-code=CHECKOUT__ASYNC_PAYMENT_PROCESS_INTERRUPTED';

      $res = $this->mcp->bookPrepay($userData, $config);
      if ($this->mcp->debug && !empty($res)) $this->mcp->log->log(__CLASS__.'->'.__FUNCTION__, print_r($res, TRUE)); // log error

      if ($res->resCreate->error == 0) { // create payment
        if ($res->resCreate->status == 'INIT') { // init payment
          $this->setStatus($salesChannelContext, $trxID, $userData['order_id'], $res);

          // jump to thank you page
          $redirectUrl = urldecode($userData['respurl']);
        } else {
          $redirectUrl = urldecode($errorURL);
        }
      } else {
        $redirectUrl = urldecode($errorURL);
      }


    } catch (\Exception $e) {
      if ($this->mcp->debug) $this->mcp->log->log(__CLASS__.'->'.__FUNCTION__, $e->getMessage().' - '.$e->getCode()); // log error
      if ($this->mcp->debug && !empty($res)) $this->mcp->log->log(__CLASS__.'->'.__FUNCTION__, print_r($res, TRUE)); // log error

      throw new AsyncPaymentProcessException(
        $transaction->getOrderTransaction()->getId(),
        'An error occurred during the communication with external payment gateway'.PHP_EOL.$e->getMessage()
      );
    }

    // Redirect to external gateway
    return new RedirectResponse($redirectUrl);
  }

  /**
   * setStatus
   *
   * @param \Shopware\Core\System\SalesChannel\SalesChannelContext $SalesChannelContext
   */
  private function setStatus($SalesChannelContext, $trxid, $orderId, $res)
  {
    $comment = $res->prepayTextDE;
    if (strtolower($res->custObj->lang) != 'de') {
      $comment = $res->prepayTextEN;
    }

    $context = $SalesChannelContext->getContext();
    $state   = 'processUnconfirmed';

    $this->psh->setState($trxid, $state, $context);

    $fields = array(
      'mcpay_authid'   => $res->resCreate->payToken,
      'mcpay_trxid'    => $trxid,
      'mcpay_comments' => $comment,
    );
    $this->psh->setCustomFields($orderId, $fields, $context);
  }
}
