<?php declare(strict_types=1);

namespace Micropayment\Service;

use Micropayment\Helper\McPayment;
use Shopware\Core\Checkout\Order\Aggregate\OrderTransaction\OrderTransactionStateHandler;
use Shopware\Core\Checkout\Payment\Cart\AsyncPaymentTransactionStruct;
use Shopware\Core\Checkout\Payment\Cart\PaymentHandler\AsynchronousPaymentHandlerInterface;
use Shopware\Core\Checkout\Payment\Exception\AsyncPaymentProcessException;
use Shopware\Core\Checkout\Payment\Exception\CustomerCanceledAsyncPaymentException;
use Shopware\Core\Framework\DataAbstractionLayer\EntityRepository;
use Shopware\Core\Framework\Validation\DataBag\RequestDataBag;
use Shopware\Core\System\Locale\LocaleEntity;
use Shopware\Core\System\SalesChannel\SalesChannelContext;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpFoundation\Request;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Criteria;
use Shopware\Core\System\Language\LanguageCollection;
use Shopware\Core\Framework\Context;

class WebPay implements AsynchronousPaymentHandlerInterface
{
    /**
     * @var \Shopware\Core\Checkout\Order\Aggregate\OrderTransaction\OrderTransactionStateHandler
     */
    private $transactionStateHandler;

    /** @var \Micropayment\Service\Config */
    protected $config;

    /** @var \Micropayment\Helper\McPayment */
    protected $mcp;

    /** @var \Shopware\Core\Framework\DataAbstractionLayer\EntityRepository */
    protected $localeRepo;

    protected $paymethod;

    /**
     * WebPay constructor.
     *
     * @param \Shopware\Core\Checkout\Order\Aggregate\OrderTransaction\OrderTransactionStateHandler $transactionStateHandler
     * @param \Micropayment\Service\Config                                                          $config
     * @param \Micropayment\Helper\McPayment                                                        $mcp
     * @param \Shopware\Core\Framework\DataAbstractionLayer\EntityRepository               $localeRepo
     */
    public function __construct(
        OrderTransactionStateHandler $transactionStateHandler,
        Config $config,
        McPayment $mcp,
        EntityRepository $localeRepo
    )
    {
        $this->transactionStateHandler = $transactionStateHandler;
        $this->config = $config;
        $this->mcp = $mcp;
        $this->localeRepo = $localeRepo;
    }

    /**
     * pay
     *
     * @param \Shopware\Core\Checkout\Payment\Cart\AsyncPaymentTransactionStruct $transaction
     * @param \Shopware\Core\Framework\Validation\DataBag\RequestDataBag         $dataBag
     * @param \Shopware\Core\System\SalesChannel\SalesChannelContext             $salesChannelContext
     *
     * @return \Symfony\Component\HttpFoundation\RedirectResponse
     */
    public function pay(
        AsyncPaymentTransactionStruct $transaction,
        RequestDataBag $dataBag,
        SalesChannelContext $salesChannelContext
    ): RedirectResponse {
        try {
            $settings = $this->config->getSettings($salesChannelContext->getSalesChannel()->getId());
            $this->mcp->init($settings);

            $order = $transaction->getOrder();
            $userData = $this->mcp->getUserData($order, $settings, $salesChannelContext);

            $langISO = $this->getLocaleCodeFromContext($order->getLanguageId(), $salesChannelContext);
            $userData['lang'] = $langISO;
            $trxID = $transaction->getOrderTransaction()->getId();
            $userData['trxid'] = $trxID;
            $userData['oid'] = $order->getId();
            $userData['respurl'] = urlencode($transaction->getReturnUrl());

            $p = explode('_', $this->paymethod);
            $activePayment = strtolower($p[0]);
            $redirectUrl = $this->mcp->getURL($userData, $activePayment);

        } catch (\Exception $e) {
            throw new AsyncPaymentProcessException(
                $transaction->getOrderTransaction()->getId(),
                'An error occurred during the communication with external payment gateway' . PHP_EOL . $e->getMessage()
            );
        }

        // Redirect to external gateway
        return new RedirectResponse($redirectUrl);
    }

    /**
     * finalize
     *
     * @param \Shopware\Core\Checkout\Payment\Cart\AsyncPaymentTransactionStruct $transaction
     * @param \Symfony\Component\HttpFoundation\Request                          $request
     * @param \Shopware\Core\System\SalesChannel\SalesChannelContext             $salesChannelContext
     */
    public function finalize(
        AsyncPaymentTransactionStruct $transaction,
        Request $request,
        SalesChannelContext $salesChannelContext
    ): void {}

    /**
     * getLocaleCodeFromContext
     *
     * @param String                                                 $languageId
     * @param \Shopware\Core\System\SalesChannel\SalesChannelContext $salesChannelContext
     *
     * @return string iso2
     */
    public function getLocaleCodeFromContext(String $languageId, SalesChannelContext $salesChannelContext): string
    {
        $criteria = new Criteria([$languageId]);
        $criteria->addAssociation('locale');

        $context = $salesChannelContext->getContext();
        $localeRepository = $this->localeRepo;
        /** @var LanguageCollection $languageCollection */
        $languageCollection = $localeRepository->search($criteria, $context)->getEntities();

        $language = $languageCollection->get($languageId);
        if ($language === null) {
            return 'en';
        }

        $locale = $language->getLocale();
        if (!$locale) {
            return 'en';
        }

        return substr($locale->getCode(), 0, 2);
    }
}
